<?php
/*
Plugin Name: Image & Video XML Sitemap
Description: Enhance your website's media SEO by creating separate sitemaps for images and videos. Fully compatible with Yoast SEO, with advanced customization options.
Version: 1.0.1
Requires at least: 5.2
Requires PHP: 7.4
Text Domain: image-video-xml-sitemap
Domain Path: /languages
Author: Harpalsinh Parmar
Author URI: https://profiles.wordpress.org/developer1998/
License: GPL2
*/

if (!defined('ABSPATH')) {
    exit;
}

// Include necessary files
require_once plugin_dir_path(__FILE__) . 'includes/class-image-video-xml-sitemap.php';

// Initialize the plugin
function ivxs_init() {
    new ivxs_ImageVideoSitemap();
}
add_action('plugins_loaded', 'ivxs_init');

//Activation hook
register_activation_hook(__FILE__, function() {
    (new ivxs_ImageVideoSitemap())->ivxs_add_sitemap_rewrites();
    flush_rewrite_rules();
});

//Deactivation hook
register_deactivation_hook(__FILE__, function() {
    flush_rewrite_rules();
});

// Hook into WordPress admin initialization to check for SEO plugin.
add_action( 'admin_init', 'ivxs_check_seo_dependency' );
function ivxs_check_seo_dependency() {
    if ( is_admin() && current_user_can( 'activate_plugins' ) ) {
        if ( !is_plugin_active( 'wordpress-seo/wp-seo.php' )) {
            add_action( 'admin_notices', 'ivxs_seo_missing_notice' );
        }
        else if(empty(WPSEO_Options::get('enable_xml_sitemap'))){
            add_action( 'admin_notices', 'ivxs_seo_missing_sitemap_notice' );
        }
    }
}
function ivxs_seo_missing_notice() {
    ?>
    <div class="notice notice-error">
        <p>
            <strong><?php esc_html_e( 'Image & Video XML Sitemap:', 'image-video-xml-sitemap' ); ?></strong>
            <?php esc_html_e( 'Requires Yoast SEO to be installed and activated. Please install and activate Yoast SEO to use this plugin.', 'image-video-xml-sitemap' ); ?>
        </p>
    </div>
    <?php
}

function ivxs_seo_missing_sitemap_notice() {
    $current_screen = get_current_screen();
    if ( $current_screen && $current_screen->id === 'tools_page_image-video-xml-sitemap' ) {
    ?>
    <div class="notice notice-error">
        <p>
            <strong><?php esc_html_e( 'Image & Video XML Sitemap:', 'image-video-xml-sitemap' ); ?></strong>
            Please enable the <a href="<?php echo esc_url( admin_url( 'admin.php?page=wpseo_page_settings#/site-features' ) ); ?>">Yoast SEO XML Sitemap</a> feature to work with this plugin.
        </p>
    </div>
    <?php
    }
}