<?php
if (!defined('ABSPATH')) {
    exit;
}

class ivxs_ImageVideoSitemap {

    public function __construct(){
        add_action('admin_menu', [$this, 'ivxs_admin_menu']);
        add_action('admin_enqueue_scripts', [$this, 'ivxs_enqueue_scripts']);
        add_action('admin_post_save_ivxs_settings', [$this, 'ivxs_save_settings']);
        add_action('wpseo_sitemap_index', [$this, 'ivxs_add_index_sitemap']);
        add_action('init', [$this, 'ivxs_add_sitemap_rewrites']);
        add_action('query_vars', [$this, 'ivxs_add_query_vars']);
        add_action('template_redirect', [$this, 'ivxs_generate_custom_sitemap']);
    }

    public function ivxs_admin_menu() {
        add_submenu_page(
            'tools.php',
            __('Image & Video XML Sitemap', 'image-video-xml-sitemap'), 
            __('Image & Video XML Sitemap', 'image-video-xml-sitemap'),  
            'manage_options',
            'image-video-xml-sitemap',  
            [$this, 'ivxs_dashboard']
        );
    }

    public function ivxs_enqueue_scripts($hook) {
        if ('tools_page_image-video-xml-sitemap' === $hook) {
            wp_enqueue_style('ivxs-style', plugin_dir_url(__FILE__).'../assets/css/ivxs-admin-css.css', [], filemtime( plugin_dir_url(__FILE__) . 'assets/css/ivxs-admin-css'));

            wp_enqueue_script('jquery');
            wp_enqueue_script('ivxs-script', plugin_dir_url(__FILE__).'../assets/js/ivxs-admin-js.js', 'jquery', filemtime( plugin_dir_url(__FILE__) . 'assets/js/ivxs-admin-js.js'), true);
            wp_localize_script('ivxs-script', 'ivxs_ajax_obj', [
                'ajax_url' => admin_url('admin-ajax.php'), 'nonce' => wp_create_nonce('ivxs_nonce')
            ]);
        }
    }

    public function ivxs_dashboard() {
        ?>
        <div class="wrap">
            <h1 class="ivxs-title"><?php esc_html_e( 'Image & Video XML Sitemap', 'image-video-xml-sitemap' ); ?></h1>
            <div class="ivxs-row">
                <div class="ivxs-col-8">
                    <form method="post" action="<?php echo esc_url(admin_url('admin-post.php')); ?>">
                        <input type="hidden" name="action" value="save_ivxs_settings">
                        <?php wp_nonce_field('ivxs_save_settings', 'ivxs_nonce'); ?>
                        <div class="ivxs-global-options">
                            <h3><?php esc_html_e( 'XML Sitemaps', 'image-video-xml-sitemap' ); ?></h3>
                            <p><?php esc_html_e( "Enable the Image & Video XML sitemaps. These sitemaps are specialized files that list your website's essential media content, helping search engines discover and crawl them effectively.", 'image-video-xml-sitemap' ); ?></p>

                            <!-- Image Sitemap Toggle -->
                            <div class="ivxs-d-flex">
                                <div>
                                    <h4><?php esc_html_e( 'Image Sitemap', 'image-video-xml-sitemap' ); ?></h4>
                                    <p><?php esc_html_e( 'When you upload an image, WordPress automatically creates an image page (attachment URL) for it.', 'image-video-xml-sitemap' ); ?></p>
                                </div>
                                <label class="switch" for="image-sitemap-toggle">
                                    <input type="checkbox" id="image-sitemap-toggle" name="ivxs_enable_image_sitemap" value="1" <?php checked( get_option( 'ivxs_enable_image_sitemap', 0 ), 1 ); ?> />
                                    <div class="slider round"></div>
                                </label>
                            </div>
                            <div id="image-sitemap-options" style="display: <?php echo get_option( 'ivxs_enable_image_sitemap', 0 ) ? 'block' : 'none'; ?>;">
                                <p>
                                    <label for="image-sitemap-filename"><?php esc_html_e( 'Image Sitemap File Name:', 'image-video-xml-sitemap' ); ?></label>
                                    <input type="text" id="image-sitemap-filename" name="ivxs_image_sitemap_filename" value="<?php echo esc_attr(get_option( 'ivxs_image_sitemap_filename', '' )); ?>" class="regular-text" />
                                    <span>.xml</span>
                                </p>
                            </div>

                            <hr/>

                            <!-- Video Sitemap Toggle -->
                            <div class="ivxs-d-flex">
                                <div>
                                    <h4><?php esc_html_e( 'Video Sitemap', 'image-video-xml-sitemap' ); ?></h4>
                                    <p><?php esc_html_e( 'When you upload an video, WordPress automatically creates an video page (attachment URL) for it.', 'image-video-xml-sitemap' ); ?></p>
                                </div>
                                <label class="switch" for="video-sitemap-toggle">
                                    <input type="checkbox" id="video-sitemap-toggle" name="ivxs_enable_video_sitemap" value="1" <?php checked( get_option( 'ivxs_enable_video_sitemap', 0 ), 1 ); ?> />
                                    <div class="slider round"></div>
                                </label>
                            </div>
                            <div id="video-sitemap-options" style="display: <?php echo get_option( 'ivxs_enable_video_sitemap', 0 ) ? 'block' : 'none'; ?>;">
                                <p>
                                    <label for="video-sitemap-filename"><?php esc_html_e( 'Video Sitemap File Name:', 'image-video-xml-sitemap' ); ?></label>
                                    <input type="text" id="video-sitemap-filename" name="ivxs_video_sitemap_filename" value="<?php echo esc_attr( get_option( 'ivxs_video_sitemap_filename', '' ) ); ?>" class="regular-text" />
                                    <span>.xml</span>
                                </p>
                            </div>
                        </div>
                        <?php submit_button(); ?>
                    </form>
                    <p><em><strong>Note</strong>: Please visit Settings → Permalinks in your WordPress dashboard and simply click 'Save Changes' if the image or video XML sitemap URL is not working.</em></p>
                </div>
                <div class="ivxs-col-4">
                    <div class="ivxs-documentation">
                        <span class="dashicons dashicons-book"></span>
                        <h3><?php esc_html_e( 'Documentation', 'image-video-xml-sitemap' ); ?></h3>
                        <p><?php esc_html_e( 'Find solutions to common issues and learn how to get the most out of the plugin.', 'image-video-xml-sitemap' ); ?></p>
                        <a href="<?php echo esc_url(site_url()); ?>/wp-content/plugins/image-video-xml-sitemap/documenation.html" target="" class="read-docu"><?php esc_html_e( 'Read the Documentation', 'image-video-xml-sitemap' ); ?></a>
                    </div>
                    <div class="ivxs-help-support">
                        <span class="dashicons dashicons-sos"></span>
                        <h3><?php esc_html_e( 'Help and Support', 'image-video-xml-sitemap' ); ?></h3>
                        <p><?php esc_html_e( 'If you need assistance, visit our support forum or check the documentation for detailed guidance. Our goal is to help you achieve the best performance for your site.', 'image-video-xml-sitemap' ); ?></p>
                        <a href="https://wordpress.org/support/plugin/image-video-xml-sitemap/" target="_blank" class="read-docu"><?php esc_html_e( 'Support Forum', 'image-video-xml-sitemap' ); ?></a>
                    </div>
                </div>
            </div>
        </div>
        <?php
    }

    public function ivxs_save_settings() {
        if (!is_admin() || !current_user_can('manage_options') || !check_admin_referer('ivxs_save_settings', 'ivxs_nonce')) {
            wp_die(esc_html(__('You are not allowed to perform this action.', 'image-video-xml-sitemap')));
        }

        // Save the settings
        $enable_image_sitemap = isset($_POST['ivxs_enable_image_sitemap']) ? 1 : 0;
        $enable_video_sitemap = isset($_POST['ivxs_enable_video_sitemap']) ? 1 : 0;

        update_option('ivxs_enable_image_sitemap', $enable_image_sitemap);
        update_option('ivxs_enable_video_sitemap', $enable_video_sitemap);

        $image_sitemap_filename = isset($_POST['ivxs_image_sitemap_filename']) 
        ? sanitize_text_field(wp_unslash($_POST['ivxs_image_sitemap_filename'])) 
        : '';

        $video_sitemap_filename = isset($_POST['ivxs_video_sitemap_filename']) 
            ? sanitize_text_field(wp_unslash($_POST['ivxs_video_sitemap_filename'])) 
            : '';

        //Removed image sitemap name when it is disable
        if($enable_image_sitemap == 1){
            update_option('ivxs_image_sitemap_filename', str_replace(".xml","",$image_sitemap_filename));
        }
        else {
            update_option('ivxs_image_sitemap_filename', '');
        }

        if($enable_image_sitemap == 1 && empty($image_sitemap_filename)){
            update_option('ivxs_image_sitemap_filename', 'image');
        }

        //Removed video sitemap name when it is disable
        if($enable_video_sitemap == 1){
            update_option('ivxs_video_sitemap_filename', str_replace(".xml","",$video_sitemap_filename));
        }
        else {
            update_option('ivxs_video_sitemap_filename', '');
        }

        if($enable_video_sitemap == 1 && empty($video_sitemap_filename)){
            update_option('ivxs_video_sitemap_filename', 'video');
        }

        wp_redirect(add_query_arg(['page' => 'image-video-xml-sitemap', 'message' => 'settings_saved'], admin_url('tools.php')));
        exit;
    }

    public function ivxs_add_index_sitemap($sitemaps){

        $image_sitemap_filename = get_option('ivxs_image_sitemap_filename', '');
        $video_sitemap_filename = get_option('ivxs_video_sitemap_filename', '');
        $enable_image_sitemap = get_option('ivxs_enable_image_sitemap');
        $enable_video_sitemap = get_option('ivxs_enable_video_sitemap');

        $last_mod_date_image = $this->ivxs_get_last_uploaded_media_date('image');
        $last_mod_date_video = $this->ivxs_get_last_uploaded_media_date('video');

        if(!empty($enable_image_sitemap) && !empty($image_sitemap_filename)){
            $sitemaps .= '
            <sitemap>
                <loc>' . esc_url( site_url( '/'.$image_sitemap_filename.'.xml' ) ) . '</loc>
                <lastmod>' . esc_html( $last_mod_date_image ) . '</lastmod>
            </sitemap> ';
        }
        else {
            delete_option('ivxs_image_sitemap_filename');
        }

        if(!empty($enable_video_sitemap) && !empty($video_sitemap_filename)){
            $sitemaps .= '
                <sitemap>
                    <loc>' . esc_url( site_url( '/'.$video_sitemap_filename.'.xml' ) ) . '</loc>
                    <lastmod>' . esc_html( $last_mod_date_video ) . '</lastmod>
                </sitemap>
            ';
        }
        else {
            delete_option('ivxs_video_sitemap_filename');
        }

        return $sitemaps;
    }

    public function ivxs_get_last_uploaded_media_date( $media_type ) {
        $args = [
            'post_type'      => 'attachment',
            'post_status'    => 'inherit',
            'post_mime_type' => $media_type,
            'posts_per_page' => 1,
            'orderby'        => 'date',
            'order'          => 'DESC',
        ];
    
        $query = new WP_Query( $args );
    
        if ( $query->have_posts() ) {
            $query->the_post();
            $formatted_date = $this->ivxs_get_post_datetime_string( get_the_ID() );
            wp_reset_postdata();
            return $formatted_date;
        }
    
        return current_time( 'Y-m-d\TH:i:s+00:00' );
    }

    public function ivxs_get_post_datetime_string( $post_id ) {
        $modified_date = get_the_modified_date( 'Y-m-d H:i:s', $post_id );
        $wp_timezone = wp_timezone();
    
        try {
            $datetime = new DateTime( $modified_date, $wp_timezone );
            $datetime->setTimezone( new DateTimeZone( 'UTC' ) );
            return $datetime->format( 'Y-m-d\TH:i:s+00:00' );
        } catch ( Exception $e ) {
            return gmdate( 'Y-m-d\TH:i:s+00:00' );
        }
    }

    public function ivxs_add_sitemap_rewrites() {
        $image_sitemap_filename = get_option('ivxs_image_sitemap_filename', 'image');
        $video_sitemap_filename = get_option('ivxs_video_sitemap_filename', 'video');
    
        if (!empty($image_sitemap_filename)) {
            add_rewrite_rule('^' . preg_quote($image_sitemap_filename) . '\.xml$', 'index.php?custom_sitemap=image', 'top');
        }
    
        if (!empty($video_sitemap_filename)) {
            add_rewrite_rule('^' . preg_quote($video_sitemap_filename) . '\.xml$', 'index.php?custom_sitemap=video', 'top');
        }
    }
    
    public function ivxs_add_query_vars($vars) {
        $vars[] = 'custom_sitemap';
        return $vars;
    }
    
    public function ivxs_generate_custom_sitemap() {
        $custom_sitemap = get_query_var( 'custom_sitemap' );
        if ( $custom_sitemap === 'image' ) {
            $this->ivxs_generate_media_sitemap( 'image' );
        } elseif ( $custom_sitemap === 'video' ) {
            $this->ivxs_generate_media_sitemap( 'video' );
        }
    }

    public function ivxs_generate_media_sitemap( $media_type ) {
        header( 'Content-Type: application/xml; charset=UTF-8' );
    
        $namespace = $media_type === 'image'
            ? 'xmlns:image="http://www.google.com/schemas/sitemap-image/1.1"' 
            : 'xmlns:video="http://www.google.com/schemas/sitemap-video/1.1"';
    
        echo '<?xml version="1.0" encoding="UTF-8"?>';
        echo '<?xml-stylesheet type="text/xsl" href="' . esc_url(plugin_dir_url(__FILE__).'../assets/css/ivxs-pdf-sitemap.xsl') . '"?>';
        echo '<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9" ' . $namespace . '>';
    
        $args = [
            'post_type'      => 'attachment',
            'post_mime_type' => $media_type === 'image' 
                ? [ 'image/jpeg', 'image/png' ]
                : 'video',
            'posts_per_page' => -1,
            'post_status'    => 'inherit',
        ];
    
        $query = new WP_Query( $args );
    
        if ( $query->have_posts() ) {
            while ( $query->have_posts() ) {
                $query->the_post();
                $url = wp_get_attachment_url( get_the_ID() );
                $lastmod = $this->ivxs_get_post_datetime_string( get_the_ID() );
    
                echo '<url>';
                echo '<loc>' . esc_url( $url ) . '</loc>';
                echo '<lastmod>' . esc_html( $lastmod ) . '</lastmod>';
    
                if ( $media_type === 'image' ) {
                    echo '<image:image><image:loc>' . esc_url( $url ) . '</image:loc></image:image>';
                } elseif ( $media_type === 'video' ) {
                    $thumbnail = esc_url( wp_get_attachment_url( get_post_thumbnail_id() ) );
                    echo '<video:video>';
                    echo '<video:content_loc>' . esc_url( $url ) . '</video:content_loc>';
                    if ( $thumbnail ) {
                        echo '<video:thumbnail_loc>' . esc_url($thumbnail) . '</video:thumbnail_loc>';
                    }
                    echo '</video:video>';
                }
    
                echo '</url>';
            }
        }
    
        echo '</urlset>';
        wp_reset_postdata();
        exit;
    }
}
?>